﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Documents;

namespace XamStackedSeriesExample
{
    #region Local Classes / Structs

    public class ChartDataPoint2DGrouped
    {
        public object AxisXValue { get; set; }
        public object AxisYValue { get; set; }
        public string GroupByValue { get; set; }
    }

    #endregion

    public partial class MainWindow : Window
    {
        #region Private Variables

        private Random random = null;

        #endregion

        #region Constructor

        public MainWindow()
        {
            InitializeComponent();

            random = new Random(DateTime.Now.Millisecond);
        }

        #endregion

        #region Events

        private void btnDrawChart_Click(object sender, RoutedEventArgs e)
        {
            xamDataChart.Axes.Clear();
            xamDataChart.Series.Clear();

            List<ChartDataPoint2DGrouped> chartData = getRandomData();

            // This GroupBy object will group the data based on the GroupMemberPath.  It goes through the data and groups
            // it into a proxy object with automatically generated properties.  The generated properties use the KeyMemberPath
            // in the name along with the actual property name.  So the final result of the below code will produce this:
            //
            // Proxy Object 1
            //  A_AxisXValue
            //  A_AxisYValue
            //  A_GroupByValue
            //  B_AxisXValue
            //  B_AxisYValue
            //  B_GroupByValue
            //
            // A and B in this scenario come from the KeyMemberPath.  I set this to GroupByValue which was being set to string
            // characters.
            Infragistics.GroupBy groupedData = new Infragistics.GroupBy
            {
                GroupMemberPath = "AxisXValue",
                KeyMemberPath = "GroupByValue",
                ValueMemberPath = "AxisYValue",
                ItemsSource = chartData
            };

            // Since the axis is now bound to the GroupBy object, and since it is grouped by AxisXValue, the Key for the group is going
            // to be the DateTime value for the group.  So I set the DateTimeMemberPath to "Key".
            Infragistics.Controls.Charts.CategoryDateTimeXAxis xAxis = new Infragistics.Controls.Charts.CategoryDateTimeXAxis
            {
                ItemsSource = groupedData,
                DateTimeMemberPath = "Key",
                Label = "{Key: " + System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.ShortDatePattern + "}"
            };

            Infragistics.Controls.Charts.NumericYAxis yAxis = new Infragistics.Controls.Charts.NumericYAxis 
            {
                Label = "{:n2}"
            };

            Infragistics.Controls.Charts.StackedColumnSeries chartSeries = new Infragistics.Controls.Charts.StackedColumnSeries
            {
                XAxis = xAxis,
                YAxis = yAxis,
                AutoGenerateSeries = true,
                ItemsSource = groupedData,
            };

            xamDataChart.Axes.Add(xAxis);
            xamDataChart.Axes.Add(yAxis);
            xamDataChart.Series.Add(chartSeries);
        }

        #endregion

        #region Methods

        private List<ChartDataPoint2DGrouped> getRandomData()
        {
            List<ChartDataPoint2DGrouped> randomData = new List<ChartDataPoint2DGrouped>();

            for (int i = 0; i < 500; i++)
            {
                DateTime axisXValue;
                int axisYValue;
                //bool axisYValue;
                string groupByValue;

                getRandom(out axisXValue);
                getRandom(out axisYValue);
                getRandom(out groupByValue);

                randomData.Add(new ChartDataPoint2DGrouped
                {
                    AxisXValue = axisXValue,
                    AxisYValue = axisYValue,
                    GroupByValue = groupByValue
                });
            }

            return randomData;
        }

        private void getRandom(out string value)
        {
            int randomNumber = random.Next(0, 25);
            randomNumber += 65;
            value = Convert.ToChar(randomNumber).ToString();
        }

        private void getRandom(out bool value)
        {
            value = Convert.ToBoolean(random.Next(0, 1));
        }

        private void getRandom(out DateTime value)
        {
            value = new DateTime(2015, 1, 1).AddDays(random.Next(0, 364));
        }

        private void getRandom(out int value)
        {
            value = random.Next(0, 99);
        }

        #endregion
    }
}